/*----- PROTECTED REGION ID(DsAxis.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        DsAxis.cpp
//
// description : C++ source for the DsAxis and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               DsAxis are implemented in this file.
//
// project :     DsAxis.
//
// $Author: abrami $
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <cmath>
#include <DsAxis.h>
#include <DsAxisClass.h>
#include "DsAxisHandler.h"

/*----- PROTECTED REGION END -----*/	//	DsAxis.cpp

/**
 *  DsAxis class description:
 *    Pin hole - double slit motion server
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name           |  Method name
//================================================================
//  State                  |  Inherited (no method)
//  Status                 |  Inherited (no method)
//  AxisStopH1             |  axis_stop_h1
//  AxisStopV1             |  axis_stop_v1
//  AxisStopH2             |  axis_stop_h2
//  AxisStopV2             |  axis_stop_v2
//  AxisInitH1             |  axis_init_h1
//  AxisInitV1             |  axis_init_v1
//  AxisInitH2             |  axis_init_h2
//  AxisInitV2             |  axis_init_v2
//  MotorOnH1              |  motor_on_h1
//  MotorOnV1              |  motor_on_v1
//  MotorOnH2              |  motor_on_h2
//  MotorOnV2              |  motor_on_v2
//  MotorOffH1             |  motor_off_h1
//  MotorOffV1             |  motor_off_v1
//  MotorOffH2             |  motor_off_h2
//  MotorOffV2             |  motor_off_v2
//  GetMotionStatusH1      |  get_motion_status_h1
//  GetMotionStatusV1      |  get_motion_status_v1
//  GetMotionStatusH2      |  get_motion_status_h2
//  GetMotionStatusV2      |  get_motion_status_v2
//  DSCenterHGoToPosition  |  dscenter_hgo_to_position
//  DSCenterVGoToPosition  |  dscenter_vgo_to_position
//  DSGapH                 |  dsgap_h
//  DSGapV                 |  dsgap_v
//  DSAxisStop             |  dsaxis_stop
//  DSAxisInit             |  dsaxis_init
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  AxisPositionH1       |  Tango::DevDouble	Scalar
//  AxisPositionH2       |  Tango::DevDouble	Scalar
//  AxisPositionV1       |  Tango::DevDouble	Scalar
//  AxisPositionV2       |  Tango::DevDouble	Scalar
//  IsInitialisedH1      |  Tango::DevShort	Scalar
//  IsInitialisedH2      |  Tango::DevShort	Scalar
//  IsInitialisedV1      |  Tango::DevShort	Scalar
//  IsInitialisedV2      |  Tango::DevShort	Scalar
//  DSCenterH            |  Tango::DevDouble	Scalar
//  DSCenterV            |  Tango::DevDouble	Scalar
//  DSGapH               |  Tango::DevDouble	Scalar
//  DSGapV               |  Tango::DevDouble	Scalar
//  DSAxisIsInitialised  |  Tango::DevShort	Scalar
//  History              |  Tango::DevString	Spectrum  ( max = 2000)
//================================================================

namespace DsAxis_ns
{
/*----- PROTECTED REGION ID(DsAxis::namespace_starting) ENABLED START -----*/

	//	static initializations

	/*----- PROTECTED REGION END -----*/	//	DsAxis::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : DsAxis::DsAxis()
 *	Description : Constructors for a Tango device
 *                implementing the classDsAxis
 */
//--------------------------------------------------------
DsAxis::DsAxis(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(DsAxis::constructor_1) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	DsAxis::constructor_1
}
//--------------------------------------------------------
DsAxis::DsAxis(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(DsAxis::constructor_2) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	DsAxis::constructor_2
}
//--------------------------------------------------------
DsAxis::DsAxis(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(DsAxis::constructor_3) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	DsAxis::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : DsAxis::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void DsAxis::delete_device()
{
	DEBUG_STREAM << "DsAxis::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::delete_device) ENABLED START -----*/

	for (int i=0; i<2000; i++){
	        if (attr_History_read[i] != NULL){
	                #ifdef _DBG_HISTORY
	                        cout << "Going to delete attr_History_read[" << i << "]" << endl;
	                #endif
	                delete[] attr_History_read[i];
	                attr_History_read[i] = NULL;
	        }
	}
	
	if (hist) {
	        delete hist;
	        hist = NULL;
	}

	if(h1_axis)
	{
		delete h1_axis;
		h1_axis = 0;
	}

	if(v1_axis)
	{
		delete v1_axis;
		v1_axis = 0;
	}

	if(h2_axis)
	{
		delete h2_axis;
		h2_axis = 0;
	}

	if(v2_axis)
	{
		delete v2_axis;
		v2_axis = 0;
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::delete_device
	delete[] attr_AxisPositionH1_read;
	delete[] attr_AxisPositionH2_read;
	delete[] attr_AxisPositionV1_read;
	delete[] attr_AxisPositionV2_read;
	delete[] attr_IsInitialisedH1_read;
	delete[] attr_IsInitialisedH2_read;
	delete[] attr_IsInitialisedV1_read;
	delete[] attr_IsInitialisedV2_read;
	delete[] attr_DSCenterH_read;
	delete[] attr_DSCenterV_read;
	delete[] attr_DSGapH_read;
	delete[] attr_DSGapV_read;
	delete[] attr_DSAxisIsInitialised_read;
	delete[] attr_History_read;
}

//--------------------------------------------------------
/**
 *	Method      : DsAxis::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void DsAxis::init_device()
{
	DEBUG_STREAM << "DsAxis::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::init_device_before) ENABLED START -----*/

	set_state(Tango::INIT);
	set_status("Initializing device");

	h1_axis = 0;

	v1_axis = 0;

	h2_axis = 0;

	v2_axis = 0;

	h_motion_accuracy = 0;

	v_motion_accuracy = 0;

	init_completed = false;

	/*----- PROTECTED REGION END -----*/	//	DsAxis::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_AxisPositionH1_read = new Tango::DevDouble[1];
	attr_AxisPositionH2_read = new Tango::DevDouble[1];
	attr_AxisPositionV1_read = new Tango::DevDouble[1];
	attr_AxisPositionV2_read = new Tango::DevDouble[1];
	attr_IsInitialisedH1_read = new Tango::DevShort[1];
	attr_IsInitialisedH2_read = new Tango::DevShort[1];
	attr_IsInitialisedV1_read = new Tango::DevShort[1];
	attr_IsInitialisedV2_read = new Tango::DevShort[1];
	attr_DSCenterH_read = new Tango::DevDouble[1];
	attr_DSCenterV_read = new Tango::DevDouble[1];
	attr_DSGapH_read = new Tango::DevDouble[1];
	attr_DSGapV_read = new Tango::DevDouble[1];
	attr_DSAxisIsInitialised_read = new Tango::DevShort[1];
	attr_History_read = new Tango::DevString[2000];
	//	No longer if mandatory property not set. 
	if (mandatoryNotDefined)
		return;

	/*----- PROTECTED REGION ID(DsAxis::init_device) ENABLED START -----*/

	hist = new History_ns::history(this);
	hist->set_max_sample(2000);

	for (int i=0; i<2000; i++){
	        attr_History_read[i] = new char[2];
	        sprintf(attr_History_read[i], "-");
	        #ifdef _DBG_HISTORY
	                cout << "attr_History_read[" << i << "]=" << attr_History_read[i] << "|" << endl;
	        #endif /* _DBG_HISTORY*/
	}

	if(get_state() != Tango::FAULT)
	{
		try
		{
			INFO_STREAM << "H1 axis device: " << axisH1 << endl;
			h1_axis = new DsAxisHandler(axisH1, timeout, positionAttribute, motionAccuracyAttribute);
			double h1_motion_accuracy = h1_axis->get_motion_accuracy();

			INFO_STREAM << "V1 axis device: " << axisV1 << endl;
			v1_axis = new DsAxisHandler(axisV1, timeout, positionAttribute, motionAccuracyAttribute);
			double v1_motion_accuracy = v1_axis->get_motion_accuracy();

			INFO_STREAM << "H2 axis device: " << axisH2 << endl;
			h2_axis = new DsAxisHandler(axisH2, timeout, positionAttribute, motionAccuracyAttribute);
			double h2_motion_accuracy = h2_axis->get_motion_accuracy();

			INFO_STREAM << "V2 axis device: " << axisV2 << endl;
			v2_axis = new DsAxisHandler(axisV2, timeout, positionAttribute, motionAccuracyAttribute);
			double v2_motion_accuracy = v2_axis->get_motion_accuracy();

			h_motion_accuracy = max(h1_motion_accuracy, h2_motion_accuracy) * 2;
			DEBUG_STREAM << "H1 motion accuracy: " << h1_motion_accuracy << endl;
			DEBUG_STREAM << "H2 motion accuracy: " << h2_motion_accuracy << endl;
			DEBUG_STREAM << "H motion accuracy: " << h_motion_accuracy << endl;

			v_motion_accuracy = max(v1_motion_accuracy, v2_motion_accuracy) * 2;
			DEBUG_STREAM << "V1 motion accuracy: " << v1_motion_accuracy << endl;
			DEBUG_STREAM << "V2 motion accuracy: " << v2_motion_accuracy << endl;
			DEBUG_STREAM << "V motion accuracy: " << v_motion_accuracy << endl;

			init_completed = true;
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "init_device");
		}
	}

	std::stringstream ss;
	ss << __func__ << "(done)";
	hist->push_end(ss.str());


	/*----- PROTECTED REGION END -----*/	//	DsAxis::init_device
}

//--------------------------------------------------------
/**
 *	Method      : DsAxis::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void DsAxis::get_device_property()
{
	/*----- PROTECTED REGION ID(DsAxis::get_device_property_before) ENABLED START -----*/

	//	Initialize property data members

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_device_property_before

	mandatoryNotDefined = false;

	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("AxisH1"));
	dev_prop.push_back(Tango::DbDatum("AxisV1"));
	dev_prop.push_back(Tango::DbDatum("AxisH2"));
	dev_prop.push_back(Tango::DbDatum("AxisV2"));
	dev_prop.push_back(Tango::DbDatum("Timeout"));
	dev_prop.push_back(Tango::DbDatum("MaxBackOff"));
	dev_prop.push_back(Tango::DbDatum("CenterToBorderDistance"));
	dev_prop.push_back(Tango::DbDatum("PositionAttribute"));
	dev_prop.push_back(Tango::DbDatum("MotionAccuracyAttribute"));
	dev_prop.push_back(Tango::DbDatum("DSType"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on DsAxisClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		DsAxisClass	*ds_class =
			(static_cast<DsAxisClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize AxisH1 from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  axisH1;
		else {
			//	Try to initialize AxisH1 from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  axisH1;
		}
		//	And try to extract AxisH1 value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisH1;

		//	Try to initialize AxisV1 from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  axisV1;
		else {
			//	Try to initialize AxisV1 from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  axisV1;
		}
		//	And try to extract AxisV1 value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisV1;

		//	Try to initialize AxisH2 from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  axisH2;
		else {
			//	Try to initialize AxisH2 from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  axisH2;
		}
		//	And try to extract AxisH2 value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisH2;

		//	Try to initialize AxisV2 from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  axisV2;
		else {
			//	Try to initialize AxisV2 from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  axisV2;
		}
		//	And try to extract AxisV2 value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisV2;

		//	Try to initialize Timeout from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  timeout;
		else {
			//	Try to initialize Timeout from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  timeout;
		}
		//	And try to extract Timeout value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  timeout;

		//	Try to initialize MaxBackOff from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  maxBackOff;
		else {
			//	Try to initialize MaxBackOff from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  maxBackOff;
		}
		//	And try to extract MaxBackOff value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  maxBackOff;

		//	Try to initialize CenterToBorderDistance from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  centerToBorderDistance;
		else {
			//	Try to initialize CenterToBorderDistance from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  centerToBorderDistance;
		}
		//	And try to extract CenterToBorderDistance value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  centerToBorderDistance;

		//	Try to initialize PositionAttribute from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  positionAttribute;
		else {
			//	Try to initialize PositionAttribute from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  positionAttribute;
		}
		//	And try to extract PositionAttribute value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  positionAttribute;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize MotionAccuracyAttribute from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  motionAccuracyAttribute;
		else {
			//	Try to initialize MotionAccuracyAttribute from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  motionAccuracyAttribute;
		}
		//	And try to extract MotionAccuracyAttribute value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  motionAccuracyAttribute;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize DSType from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  dSType;
		else {
			//	Try to initialize DSType from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  dSType;
		}
		//	And try to extract DSType value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  dSType;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

	}

	/*----- PROTECTED REGION ID(DsAxis::get_device_property_after) ENABLED START -----*/

	if ( !((dSType=="Padres")||(dSType=="Simple")) ){
		set_state(Tango::FAULT);
		set_status("DSType property not correct");
		return;
	}

	if (motionAccuracyAttribute.empty()){
		set_state(Tango::FAULT);
		set_status("MotionAccuracyAttribute property not defined");
		return;
	}

	if (positionAttribute.empty()){
		set_state(Tango::FAULT);
		set_status("PositionAttribute property not defined");
		return;
	}

	if (dev_prop[0].is_empty())
	{
		set_state(Tango::FAULT);
		set_status("AxisH1 property not defined");
		return;
	}

	if(axisH1.empty() || axisH1.compare("undefined")==0)
	{
		set_state(Tango::FAULT);
		set_status("AxisH1 property is empty");
		return;
	}

	if (dev_prop[1].is_empty())
	{
		set_state(Tango::FAULT);
		set_status("AxisV1 property not defined");
		return;
	}

	if(axisV1.empty() || axisV1.compare("undefined")==0)
	{
		set_state(Tango::FAULT);
		set_status("AxisV1 property is empty");
		return;
	}

	if (dev_prop[2].is_empty())
	{
		set_state(Tango::FAULT);
		set_status("AxisH2 property not defined");
		return;
	}

	if(axisH2.empty() || axisH2.compare("undefined")==0)
	{
		set_state(Tango::FAULT);
		set_status("AxisH2 property is empty");
		return;
	}

	if (dev_prop[3].is_empty())
	{
		set_state(Tango::FAULT);
		set_status("AxisV2 property not defined");
		return;
	}

	if(axisV2.empty() || axisV2.compare("undefined")==0)
	{
		set_state(Tango::FAULT);
		set_status("AxisV2 property is empty");
		return;
	}

	if (dev_prop[4].is_empty())
	{
		WARN_STREAM << "Timeout property not defined, default: " << maxBackOff << endl;
	}

	if(timeout < 1 || timeout > 10000)
	{
		set_state(Tango::FAULT);
		set_status("Timeout invalid value, valid range [1,10000]");
		return;
	}

	if (dev_prop[5].is_empty())
	{
		WARN_STREAM << "MaxBackOff property not defined, default: " << maxBackOff << endl;
	}

	if(maxBackOff < 0 || maxBackOff > 600)
	{
		set_state(Tango::FAULT);
		set_status("MaxBackOff invalid value, valid range [0,600]");
		return;
	}

	if (dev_prop[6].is_empty())
	{
		WARN_STREAM << "CenterToBorderDistance property not defined, default: " << centerToBorderDistance << endl;
	}

	if(centerToBorderDistance < 0 || centerToBorderDistance > 100)
	{
		set_state(Tango::FAULT);
		set_status("MaxBackOff invalid value, valid range [0,100]");
		return;
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_device_property_after
}
//--------------------------------------------------------
/**
 *	Method      : DsAxis::check_mandatory_property()
 *	Description : For mandatory properties check if defined in database.
 */
//--------------------------------------------------------
void DsAxis::check_mandatory_property(Tango::DbDatum &class_prop, Tango::DbDatum &dev_prop)
{
	//	Check if all properties are empty
	if (class_prop.is_empty() && dev_prop.is_empty())
	{
		TangoSys_OMemStream	tms;
		tms << endl <<"Property \'" << dev_prop.name;
		if (Tango::Util::instance()->_UseDb==true)
			tms << "\' is mandatory but not defined in database";
		else
			tms << "\' is mandatory but cannot be defined without database";
		append_status(tms.str());
		mandatoryNotDefined = true;
		/*----- PROTECTED REGION ID(DsAxis::check_mandatory_property) ENABLED START -----*/
		cerr << tms.str() << " for " << device_name << endl;
		
		/*----- PROTECTED REGION END -----*/	//	DsAxis::check_mandatory_property
	}
}


//--------------------------------------------------------
/**
 *	Method      : DsAxis::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void DsAxis::always_executed_hook()
{
	DEBUG_STREAM << "DsAxis::always_executed_hook()  " << device_name << endl;
	if (mandatoryNotDefined)
	{
		Tango::Except::throw_exception(
					(const char *)"PROPERTY_NOT_SET",
					get_status().c_str(),
					(const char *)"DsAxis::always_executed_hook()");
	}
	/*----- PROTECTED REGION ID(DsAxis::always_executed_hook) ENABLED START -----*/

#ifdef _MDM
	if(update_allowed())
	{
		try
		{
			h1_axis->update_attributes();

			v1_axis->update_attributes();

			h2_axis->update_attributes();

			v2_axis->update_attributes();

			update_state();

			update_status();
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "always_executed_hook");
		}
	}
#endif

	/*----- PROTECTED REGION END -----*/	//	DsAxis::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : DsAxis::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void DsAxis::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "DsAxis::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_attr_hardware) ENABLED START -----*/

	//	Add your own code

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : DsAxis::write_attr_hardware()
 *	Description : Hardware writing for attributes
 */
//--------------------------------------------------------
void DsAxis::write_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "DsAxis::write_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::write_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute AxisPositionH1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_AxisPositionH1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_AxisPositionH1(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_AxisPositionH1) ENABLED START -----*/

	try
	{
		*attr_AxisPositionH1_read = h1_axis->read_position();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "read_AxisPositionH1");
	}

	attr.set_value(attr_AxisPositionH1_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_AxisPositionH1
}
//--------------------------------------------------------
/**
 *	Write attribute AxisPositionH1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_AxisPositionH1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_AxisPositionH1(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_AxisPositionH1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	try
	{
		h1_axis->write_position(w_val);
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "write_AxisPositionH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_AxisPositionH1
}
//--------------------------------------------------------
/**
 *	Read attribute AxisPositionH2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_AxisPositionH2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_AxisPositionH2(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_AxisPositionH2) ENABLED START -----*/

	try
	{
		*attr_AxisPositionH2_read = h2_axis->read_position();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "read_AxisPositionH2");
	}

	attr.set_value(attr_AxisPositionH2_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_AxisPositionH2
}
//--------------------------------------------------------
/**
 *	Write attribute AxisPositionH2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_AxisPositionH2(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_AxisPositionH2(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_AxisPositionH2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	try
	{
		h2_axis->write_position(w_val);
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "write_AxisPositionH2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_AxisPositionH2
}
//--------------------------------------------------------
/**
 *	Read attribute AxisPositionV1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_AxisPositionV1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_AxisPositionV1(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_AxisPositionV1) ENABLED START -----*/

	try
	{
		*attr_AxisPositionV1_read = v1_axis->read_position();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "read_AxisPositionV1");
	}

	attr.set_value(attr_AxisPositionV1_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_AxisPositionV1
}
//--------------------------------------------------------
/**
 *	Write attribute AxisPositionV1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_AxisPositionV1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_AxisPositionV1(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_AxisPositionV1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	try
	{
		v1_axis->write_position(w_val);
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "write_AxisPositionV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_AxisPositionV1
}
//--------------------------------------------------------
/**
 *	Read attribute AxisPositionV2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_AxisPositionV2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_AxisPositionV2(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_AxisPositionV2) ENABLED START -----*/

	try
	{
		*attr_AxisPositionV2_read = v2_axis->read_position();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "read_AxisPositionV2");
	}

	attr.set_value(attr_AxisPositionV2_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_AxisPositionV2
}
//--------------------------------------------------------
/**
 *	Write attribute AxisPositionV2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_AxisPositionV2(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_AxisPositionV2(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_AxisPositionV2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	try
	{
		v2_axis->write_position(w_val);
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "write_AxisPositionV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_AxisPositionV2
}
//--------------------------------------------------------
/**
 *	Read attribute IsInitialisedH1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevShort
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_IsInitialisedH1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_IsInitialisedH1(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_IsInitialisedH1) ENABLED START -----*/

	*attr_IsInitialisedH1_read = h1_axis->is_initialised();

	attr.set_value(attr_IsInitialisedH1_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_IsInitialisedH1
}
//--------------------------------------------------------
/**
 *	Read attribute IsInitialisedH2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevShort
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_IsInitialisedH2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_IsInitialisedH2(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_IsInitialisedH2) ENABLED START -----*/

	*attr_IsInitialisedH2_read = h2_axis->is_initialised();

	attr.set_value(attr_IsInitialisedH2_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_IsInitialisedH2
}
//--------------------------------------------------------
/**
 *	Read attribute IsInitialisedV1 related method
 *	Description: 
 *
 *	Data type:	Tango::DevShort
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_IsInitialisedV1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_IsInitialisedV1(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_IsInitialisedV1) ENABLED START -----*/

	*attr_IsInitialisedV1_read = v1_axis->is_initialised();

	attr.set_value(attr_IsInitialisedV1_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_IsInitialisedV1
}
//--------------------------------------------------------
/**
 *	Read attribute IsInitialisedV2 related method
 *	Description: 
 *
 *	Data type:	Tango::DevShort
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_IsInitialisedV2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_IsInitialisedV2(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_IsInitialisedV2) ENABLED START -----*/

	*attr_IsInitialisedV2_read = v2_axis->is_initialised();

	attr.set_value(attr_IsInitialisedV2_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_IsInitialisedV2
}
//--------------------------------------------------------
/**
 *	Read attribute DSCenterH related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_DSCenterH(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_DSCenterH(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_DSCenterH) ENABLED START -----*/

	Tango::DevDouble h1_position = 0;

	Tango::DevDouble h2_position = 0;

	if ( dSType=="Padres" ){
		try
		{
			h1_position = h1_axis->read_position();
	
			h2_position = h2_axis->read_position();
	
			*attr_DSCenterH_read = calculate_center( h1_position, h2_position );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSCenterH");
		}
	
		attr.set_value(attr_DSCenterH_read);
	
		if( std::fabs( h1_position - h2_position ) > (centerToBorderDistance * 2 + h_motion_accuracy) )
			attr.set_quality(Tango::ATTR_WARNING);
	} else

	if ( dSType=="Simple" ){
		// h1 = left
		// h2 = right
		try {
			h1_position = h1_axis->read_position();
			h2_position = h2_axis->read_position();
			*attr_DSCenterH_read = (h1_position - h2_position)/2 + h2_position ;
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSCenterH");
		}
	
		attr.set_value(attr_DSCenterH_read);
	} else
	{
		*attr_DSCenterH_read = nan("");
		attr.set_value(attr_DSCenterH_read);
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_DSCenterH
}
//--------------------------------------------------------
/**
 *	Write attribute DSCenterH related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_DSCenterH(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_DSCenterH(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_DSCenterH) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	if ( dSType=="Padres" ){
		try
		{
			Tango::DevDouble h1_position = h1_axis->read_position();
	
			Tango::DevDouble h2_position = h2_axis->read_position();
	
			Tango::DevDouble delta = w_val - calculate_center( h1_position, h2_position );
	
			h1_axis->write_position( h1_position + delta );
	
			h2_axis->write_position( h2_position + delta );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSCenterH");
		}
	} else

	if ( dSType=="Simple" ){
		// h1 = left
		// h2 = right
		try
		{
			Tango::DevDouble h1_position = h1_axis->read_position();
	
			Tango::DevDouble h2_position = h2_axis->read_position();
	
			Tango::DevDouble delta = w_val - ((h1_position - h2_position)/2 + h2_position);
	
			h1_axis->write_position( h1_position + delta );
	
			h2_axis->write_position( h2_position + delta );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSCenterH");
		}
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_DSCenterH
}
//--------------------------------------------------------
/**
 *	Read attribute DSCenterV related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_DSCenterV(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_DSCenterV(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_DSCenterV) ENABLED START -----*/

	Tango::DevDouble v1_position = 0;

	Tango::DevDouble v2_position = 0;

	if ( dSType=="Padres" ){
		try
		{
			v1_position = v1_axis->read_position();
	
			v2_position = v2_axis->read_position();
	
			*attr_DSCenterV_read = calculate_center( v1_position, v2_position );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSCenterV");
		}
	
		attr.set_value(attr_DSCenterV_read);
	
		if( std::fabs( v1_position - v2_position ) > (centerToBorderDistance * 2 + v_motion_accuracy) )
			attr.set_quality(Tango::ATTR_WARNING);
	} else

	if ( dSType=="Simple" ){
		// v1 = up
		// v2 = down
		try {
			v1_position = v1_axis->read_position();
			v2_position = v2_axis->read_position();
			*attr_DSCenterV_read = (v1_position - v2_position)/2 + v2_position ;
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSCenterV");
		}
	
		attr.set_value(attr_DSCenterV_read);
	} else
	{
		*attr_DSCenterV_read = nan("");
		attr.set_value(attr_DSCenterV_read);
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_DSCenterV
}
//--------------------------------------------------------
/**
 *	Write attribute DSCenterV related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_DSCenterV(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_DSCenterV(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_DSCenterV) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	if ( dSType=="Padres" ){
		try
		{
			Tango::DevDouble v1_position = v1_axis->read_position();
	
			Tango::DevDouble v2_position = v2_axis->read_position();
	
			Tango::DevDouble delta = w_val - calculate_center( v1_position, v2_position );
	
			v1_axis->write_position( v1_position + delta );
	
			v2_axis->write_position( v2_position + delta );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSCenterV");
		}
	} else

	if ( dSType=="Simple" ){
		// v1 = up
		// v2 = down
		try
		{
			Tango::DevDouble v1_position = v1_axis->read_position();
	
			Tango::DevDouble v2_position = v2_axis->read_position();
	
			Tango::DevDouble delta = w_val - ((v1_position - v2_position)/2 + v2_position);
	
			v1_axis->write_position( v1_position + delta );
	
			v2_axis->write_position( v2_position + delta );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSCenterV");
		}
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_DSCenterV
}
//--------------------------------------------------------
/**
 *	Read attribute DSGapH related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_DSGapH(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_DSGapH(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_DSGapH) ENABLED START -----*/

	Tango::DevDouble h1_position = 0;

	Tango::DevDouble h2_position = 0;

	if ( dSType=="Padres" ){
		try
		{
			h1_position = h1_axis->read_position();
	
			h2_position = h2_axis->read_position();
	
			*attr_DSGapH_read = calculate_gap( h1_position, h2_position );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSGapH");
		}
	
		attr.set_value(attr_DSGapH_read);
	
		if( *attr_DSGapH_read < 0 )
			attr.set_quality(Tango::ATTR_WARNING, true);
	
		if( (h2_position - h1_position) > h_motion_accuracy )
			attr.set_quality(Tango::ATTR_WARNING, true);
	} else

	if ( dSType=="Simple" ){
		// h1 = left
		// h2 = right
		try {
			h1_position = h1_axis->read_position();
			h2_position = h2_axis->read_position();
			*attr_DSGapH_read = h1_position - h2_position;
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSGapH");
		}
	
		attr.set_value(attr_DSGapH_read);
	} else
	{
		*attr_DSGapH_read = nan("");
		attr.set_value(attr_DSGapH_read);
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_DSGapH
}
//--------------------------------------------------------
/**
 *	Write attribute DSGapH related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_DSGapH(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_DSGapH(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_DSGapH) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	if ( dSType=="Padres" ){
		if(w_val < 0 || w_val > 20)
			Tango::Except::throw_exception("","Gap invalid value, valid[0-20]","write_DSGapH");
	
		try
		{
			Tango::DevDouble h1_position = h1_axis->read_position();
	
			Tango::DevDouble h2_position = h2_axis->read_position();
	
			Tango::DevDouble delta = w_val - calculate_gap( h1_position, h2_position );
	
			if( (h2_position - h1_position) > h_motion_accuracy )
			{
				h1_axis->write_position( h1_position + delta/2 );
	
				h2_axis->write_position( h2_position - delta/2 );
			}
			else
			{
				h1_axis->write_position( h1_position - delta/2 );
	
				h2_axis->write_position( h2_position + delta/2 );
			}
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSGapH");
		}
	} else

	if ( dSType=="Simple" ){
		try
		{
			Tango::DevDouble h1_position = h1_axis->read_position();
			Tango::DevDouble h2_position = h2_axis->read_position();
	
			Tango::DevDouble centerH = (h1_position - h2_position)/2 + h2_position ;
	
			h1_axis->write_position( centerH + w_val/2 );
			h2_axis->write_position( centerH - w_val/2 );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSGapH");
		}
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_DSGapH
}
//--------------------------------------------------------
/**
 *	Read attribute DSGapV related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_DSGapV(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_DSGapV(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_DSGapV) ENABLED START -----*/

	Tango::DevDouble v1_position = 0;

	Tango::DevDouble v2_position = 0;

	if ( dSType=="Padres" ){
		try
		{
			v1_position = v1_axis->read_position();
	
			v2_position = v2_axis->read_position();
	
			*attr_DSGapV_read = calculate_gap( v1_axis->read_position(), v2_axis->read_position() );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSGapV");
		}
	
		attr.set_value(attr_DSGapV_read);
	
		if( *attr_DSGapV_read < 0 )
			attr.set_quality(Tango::ATTR_WARNING);
	
		if( (v2_position - v1_position) > v_motion_accuracy )
			attr.set_quality(Tango::ATTR_WARNING);
	} else

	if ( dSType=="Simple" ){
		// v1 = up
		// v2 = down
		try {
			v1_position = v1_axis->read_position();
			v2_position = v2_axis->read_position();
			*attr_DSGapV_read = v1_position - v2_position;
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSGapV");
		}
	
		attr.set_value(attr_DSGapV_read);
	} else
	{
		*attr_DSGapV_read = nan("");
		attr.set_value(attr_DSGapV_read);
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_DSGapV
}
//--------------------------------------------------------
/**
 *	Write attribute DSGapV related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::write_DSGapV(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "DsAxis::write_DSGapV(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(DsAxis::write_DSGapV) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << w_val << ")";
	hist->push_end(ss.str());

	if ( dSType=="Padres" ){
		if(w_val < 0 || w_val > 20)
			Tango::Except::throw_exception("","Gap invalid value, valid[0-20]","write_DSGapV");
	
		try
		{
			Tango::DevDouble v1_position = v1_axis->read_position();
	
			Tango::DevDouble v2_position = v2_axis->read_position();
	
			Tango::DevDouble delta = w_val - calculate_gap( v1_position, v2_position );
	
			if( (v2_position - v1_position) > v_motion_accuracy )
			{
				v1_axis->write_position( v1_position + delta/2 );
	
				v2_axis->write_position( v2_position - delta/2 );
			}
			else
			{
				v1_axis->write_position( v1_position - delta/2 );
	
				v2_axis->write_position( v2_position + delta/2 );
			}
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSCenterH");
		}
	} else

	if ( dSType=="Simple" ){
		try
		{
			Tango::DevDouble v1_position = v1_axis->read_position();
			Tango::DevDouble v2_position = v2_axis->read_position();
	
			Tango::DevDouble centerV = (v1_position - v2_position)/2 + v2_position ;
	
			v1_axis->write_position( centerV + w_val/2 );
			v2_axis->write_position( centerV - w_val/2 );
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "write_DSGapV");
		}
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::write_DSGapV
}
//--------------------------------------------------------
/**
 *	Read attribute DSAxisIsInitialised related method
 *	Description: 
 *
 *	Data type:	Tango::DevShort
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void DsAxis::read_DSAxisIsInitialised(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_DSAxisIsInitialised(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_DSAxisIsInitialised) ENABLED START -----*/

#ifndef _MDM
	if(update_allowed())
	{
		try
		{
			h1_axis->update_attributes();

			v1_axis->update_attributes();

			h2_axis->update_attributes();

			v2_axis->update_attributes();

			update_state();

			update_status();
		}
		catch(Tango::DevFailed& ex)
		{
			exception_handler(ex, Tango::FAULT, "read_DSAxisIsInitialised");
		}
	}
#endif

	*attr_DSAxisIsInitialised_read =
			h1_axis->is_initialised() &&
			v1_axis->is_initialised() &&
			h2_axis->is_initialised() &&
			v2_axis->is_initialised();

	attr.set_value(attr_DSAxisIsInitialised_read);

	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_DSAxisIsInitialised
}
//--------------------------------------------------------
/**
 *	Read attribute History related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Spectrum max = 2000
 */
//--------------------------------------------------------
void DsAxis::read_History(Tango::Attribute &attr)
{
	DEBUG_STREAM << "DsAxis::read_History(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(DsAxis::read_History) ENABLED START -----*/

	int howmany = hist->copyto(attr_History_read);
	if (howmany > 0) attr.set_value(attr_History_read, howmany);

	//	Set the attribute value
	//attr.set_value(attr_History_read, 2000);
	
	/*----- PROTECTED REGION END -----*/	//	DsAxis::read_History
}

//--------------------------------------------------------
/**
 *	Method      : DsAxis::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void DsAxis::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(DsAxis::add_dynamic_attributes) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic attributes if any
	
	/*----- PROTECTED REGION END -----*/	//	DsAxis::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command AxisStopH1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_stop_h1()
{
	DEBUG_STREAM << "DsAxis::AxisStopH1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_stop_h1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h1_axis->axis_stop();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisStopH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_stop_h1
}
//--------------------------------------------------------
/**
 *	Command AxisStopV1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_stop_v1()
{
	DEBUG_STREAM << "DsAxis::AxisStopV1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_stop_v1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v1_axis->axis_stop();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisStopV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_stop_v1
}
//--------------------------------------------------------
/**
 *	Command AxisStopH2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_stop_h2()
{
	DEBUG_STREAM << "DsAxis::AxisStopH2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_stop_h2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h2_axis->axis_stop();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisStopH2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_stop_h2
}
//--------------------------------------------------------
/**
 *	Command AxisStopV2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_stop_v2()
{
	DEBUG_STREAM << "DsAxis::AxisStopV2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_stop_v2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v2_axis->axis_stop();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisStopV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_stop_v2
}
//--------------------------------------------------------
/**
 *	Command AxisInitH1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_init_h1()
{
	DEBUG_STREAM << "DsAxis::AxisInitH1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_init_h1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h1_axis->axis_init();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisInitH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_init_h1
}
//--------------------------------------------------------
/**
 *	Command AxisInitV1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_init_v1()
{
	DEBUG_STREAM << "DsAxis::AxisInitV1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_init_v1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v1_axis->axis_init();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisInitV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_init_v1
}
//--------------------------------------------------------
/**
 *	Command AxisInitH2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_init_h2()
{
	DEBUG_STREAM << "DsAxis::AxisInitH2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_init_h2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h2_axis->axis_init();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "axis_init_h2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_init_h2
}
//--------------------------------------------------------
/**
 *	Command AxisInitV2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::axis_init_v2()
{
	DEBUG_STREAM << "DsAxis::AxisInitV2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::axis_init_v2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v2_axis->axis_init();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "AxisInitV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::axis_init_v2
}
//--------------------------------------------------------
/**
 *	Command MotorOnH1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_on_h1()
{
	DEBUG_STREAM << "DsAxis::MotorOnH1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_on_h1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h1_axis->motor_on();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOnH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_on_h1
}
//--------------------------------------------------------
/**
 *	Command MotorOnV1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_on_v1()
{
	DEBUG_STREAM << "DsAxis::MotorOnV1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_on_v1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v1_axis->motor_on();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOnV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_on_v1
}
//--------------------------------------------------------
/**
 *	Command MotorOnH2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_on_h2()
{
	DEBUG_STREAM << "DsAxis::MotorOnH2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_on_h2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h2_axis->motor_on();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOnH2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_on_h2
}
//--------------------------------------------------------
/**
 *	Command MotorOnV2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_on_v2()
{
	DEBUG_STREAM << "DsAxis::MotorOnV2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_on_v2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v2_axis->motor_on();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOnV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_on_v2
}
//--------------------------------------------------------
/**
 *	Command MotorOffH1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_off_h1()
{
	DEBUG_STREAM << "DsAxis::MotorOffH1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_off_h1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h1_axis->motor_off();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOffH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_off_h1
}
//--------------------------------------------------------
/**
 *	Command MotorOffV1 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_off_v1()
{
	DEBUG_STREAM << "DsAxis::MotorOffV1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_off_v1) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v1_axis->motor_off();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOffV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_off_v1
}
//--------------------------------------------------------
/**
 *	Command MotorOffH2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_off_h2()
{
	DEBUG_STREAM << "DsAxis::MotorOffH2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_off_h2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		h2_axis->motor_off();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOffH2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_off_h2
}
//--------------------------------------------------------
/**
 *	Command MotorOffV2 related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::motor_off_v2()
{
	DEBUG_STREAM << "DsAxis::MotorOffV2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::motor_off_v2) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		v2_axis->motor_off();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "MotorOffV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::motor_off_v2
}
//--------------------------------------------------------
/**
 *	Command GetMotionStatusH1 related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevLong DsAxis::get_motion_status_h1()
{
	Tango::DevLong argout;
	DEBUG_STREAM << "DsAxis::GetMotionStatusH1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::get_motion_status_h1) ENABLED START -----*/

	try
	{
		argout = h1_axis->get_motion_status();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "GetMotionStatusH1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_motion_status_h1
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetMotionStatusV1 related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevLong DsAxis::get_motion_status_v1()
{
	Tango::DevLong argout;
	DEBUG_STREAM << "DsAxis::GetMotionStatusV1()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::get_motion_status_v1) ENABLED START -----*/

	try
	{
		argout = v1_axis->get_motion_status();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "GetMotionStatusV1");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_motion_status_v1
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetMotionStatusH2 related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevLong DsAxis::get_motion_status_h2()
{
	Tango::DevLong argout;
	DEBUG_STREAM << "DsAxis::GetMotionStatusH2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::get_motion_status_h2) ENABLED START -----*/

	try
	{
		argout = h2_axis->get_motion_status();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "GetMotionStatusH2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_motion_status_h2
	return argout;
}
//--------------------------------------------------------
/**
 *	Command GetMotionStatusV2 related method
 *	Description: 
 *
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevLong DsAxis::get_motion_status_v2()
{
	Tango::DevLong argout;
	DEBUG_STREAM << "DsAxis::GetMotionStatusV2()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::get_motion_status_v2) ENABLED START -----*/

	try
	{
		argout = v2_axis->get_motion_status();
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "GetMotionStatusV2");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::get_motion_status_v2
	return argout;
}
//--------------------------------------------------------
/**
 *	Command DSCenterHGoToPosition related method
 *	Description: 
 *
 *	@param argin 
 */
//--------------------------------------------------------
void DsAxis::dscenter_hgo_to_position(Tango::DevDouble argin)
{
	DEBUG_STREAM << "DsAxis::DSCenterHGoToPosition()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dscenter_hgo_to_position) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		Tango::DevDouble h1_position = h1_axis->read_position();

		Tango::DevDouble h2_position = h2_axis->read_position();

		Tango::DevDouble delta = argin - calculate_center( h1_position, h2_position );

		h1_axis->write_position( h1_position + delta );

		h2_axis->write_position( h2_position + delta );
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "DSCenterHGoToPosition");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dscenter_hgo_to_position
}
//--------------------------------------------------------
/**
 *	Command DSCenterVGoToPosition related method
 *	Description: 
 *
 *	@param argin 
 */
//--------------------------------------------------------
void DsAxis::dscenter_vgo_to_position(Tango::DevDouble argin)
{
	DEBUG_STREAM << "DsAxis::DSCenterVGoToPosition()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dscenter_vgo_to_position) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	try
	{
		Tango::DevDouble v1_position = v1_axis->read_position();

		Tango::DevDouble v2_position = v2_axis->read_position();

		Tango::DevDouble delta = argin - calculate_center( v1_position, v2_position );

		v1_axis->write_position( v1_position + delta );

		v2_axis->write_position( v2_position + delta );
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "DSCenterVGoToPosition");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dscenter_vgo_to_position
}
//--------------------------------------------------------
/**
 *	Command DSGapH related method
 *	Description: 
 *
 *	@param argin 
 */
//--------------------------------------------------------
void DsAxis::dsgap_h(Tango::DevDouble argin)
{
	DEBUG_STREAM << "DsAxis::DSGapH()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dsgap_h) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << argin << ")";
	hist->push_end(ss.str());

	if(argin < 0 || argin > 20)
		Tango::Except::throw_exception("","Gap invalid value, valid[0-20]","DSGapH");

	try
	{
		Tango::DevDouble h1_position = h1_axis->read_position();

		Tango::DevDouble h2_position = h2_axis->read_position();

		Tango::DevDouble delta = argin - calculate_gap( h1_position, h2_position );

		if( (h2_position - h1_position) > h_motion_accuracy )
		{
			h1_axis->write_position( h1_position + delta/2 );

			h2_axis->write_position( h2_position - delta/2 );
		}
		else
		{
			h1_axis->write_position( h1_position - delta/2 );

			h2_axis->write_position( h2_position + delta/2 );
		}
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "DSGapH");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dsgap_h
}
//--------------------------------------------------------
/**
 *	Command DSGapV related method
 *	Description: 
 *
 *	@param argin 
 */
//--------------------------------------------------------
void DsAxis::dsgap_v(Tango::DevDouble argin)
{
	DEBUG_STREAM << "DsAxis::DSGapV()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dsgap_v) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "(" << argin << ")";
	hist->push_end(ss.str());

	if(argin < 0 || argin > 20)
		Tango::Except::throw_exception("","Gap invalid value, valid[0-20]","DSGapV");

	try
	{
		Tango::DevDouble v1_position = v1_axis->read_position();

		Tango::DevDouble v2_position = v2_axis->read_position();

		Tango::DevDouble delta = argin - calculate_gap( v1_position, v2_position );

		if( (v2_position - v1_position) > v_motion_accuracy )
		{
			v1_axis->write_position( v1_position + delta/2 );

			v2_axis->write_position( v2_position - delta/2 );
		}
		else
		{
			v1_axis->write_position( v1_position - delta/2 );

			v2_axis->write_position( v2_position + delta/2 );
		}
	}
	catch(Tango::DevFailed& ex)
	{
		exception_handler(ex, Tango::FAULT, "DSGapV");
	}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dsgap_v
}
//--------------------------------------------------------
/**
 *	Command DSAxisStop related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::dsaxis_stop()
{
	DEBUG_STREAM << "DsAxis::DSAxisStop()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dsaxis_stop) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	axis_stop_h1();

	axis_stop_v1();

	axis_stop_h2();

	axis_stop_v2();

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dsaxis_stop
}
//--------------------------------------------------------
/**
 *	Command DSAxisInit related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void DsAxis::dsaxis_init()
{
	DEBUG_STREAM << "DsAxis::DSAxisInit()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(DsAxis::dsaxis_init) ENABLED START -----*/

	std::stringstream ss;
	ss << __func__ << "()";
	hist->push_end(ss.str());

	axis_init_h1();

	axis_init_v1();

	axis_init_h2();

	axis_init_v2();

	/*----- PROTECTED REGION END -----*/	//	DsAxis::dsaxis_init
}
//--------------------------------------------------------
/**
 *	Method      : DsAxis::add_dynamic_commands()
 *	Description : Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void DsAxis::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(DsAxis::add_dynamic_commands) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic commands if any
	
	/*----- PROTECTED REGION END -----*/	//	DsAxis::add_dynamic_commands
}

/*----- PROTECTED REGION ID(DsAxis::namespace_ending) ENABLED START -----*/


//========================================================
//					Private methods
//========================================================

//--------------------------------------------------------
/**
 *	Execute the update_state_allowed command:
 */
//--------------------------------------------------------
Tango::DevDouble DsAxis::calculate_center(Tango::DevDouble pos_1, Tango::DevDouble pos_2)
{
	Tango::DevDouble p1_h1 = pos_1 + centerToBorderDistance;

	Tango::DevDouble p2_h1 = pos_1 - centerToBorderDistance;

	Tango::DevDouble p1_h2 = pos_2 + centerToBorderDistance;

	Tango::DevDouble p2_h2 = pos_2 - centerToBorderDistance;

	return ( ( min(p1_h1, p1_h2) + max(p2_h1, p2_h2) ) / 2 );
}

//--------------------------------------------------------
/**
 *	Execute the update_state_allowed command:
 */
//--------------------------------------------------------
Tango::DevDouble DsAxis::calculate_gap(Tango::DevDouble pos_1, Tango::DevDouble pos_2)
{
	Tango::DevDouble p1_h1 = pos_1 + centerToBorderDistance;

	Tango::DevDouble p2_h1 = pos_1 - centerToBorderDistance;

	Tango::DevDouble p1_h2 = pos_2 + centerToBorderDistance;

	Tango::DevDouble p2_h2 = pos_2 - centerToBorderDistance;

	return ( min(p1_h1, p1_h2) - max(p2_h1, p2_h2) );

}
//--------------------------------------------------------
/**
 *	Execute the update_state_allowed command:
 */
//--------------------------------------------------------
bool DsAxis::update_allowed()
{
	DEBUG_STREAM << "DsAxis::update_allowed()  - " << device_name << endl;

	bool is_allowed = false;

	if(init_completed)
	{
		if(get_state() == Tango::FAULT)
		{
			time_t now;
			time(&now);

			double elapsed_t = difftime(now, last_exception_t);

			DEBUG_STREAM  << "Back off: " << elapsed_t << "/" << current_back_off << " : " << maxBackOff << endl;

			if(elapsed_t >= current_back_off)
			{
				is_allowed = true;
				current_back_off = current_back_off * 2;

				if(current_back_off > maxBackOff)
				{
					current_back_off = maxBackOff;
				}
			}
		}
		else
		{
			current_back_off = 1;
			is_allowed = true;
		}
	}

	return is_allowed;
}

//--------------------------------------------------------
/**
 *	Execute the update_state command:
 */
//--------------------------------------------------------
void DsAxis::update_state()
{
	Tango::DevState new_state = Tango::UNKNOWN;

#ifdef _MDM
	if(h1_axis->get_state() == Tango::OFF ||
		v1_axis->get_state() == Tango::OFF ||
		h2_axis->get_state() == Tango::OFF ||
		v2_axis->get_state()  == Tango::OFF)
	{
		new_state = Tango::OFF;
	}

	if(h1_axis->get_state() == Tango::STANDBY ||
		v1_axis->get_state() == Tango::STANDBY ||
		h2_axis->get_state() == Tango::STANDBY ||
		v2_axis->get_state()  == Tango::STANDBY)
	{
		new_state = Tango::STANDBY;
	}

	if(h1_axis->get_state() == Tango::ALARM ||
		v1_axis->get_state() == Tango::ALARM ||
		h2_axis->get_state() == Tango::ALARM ||
		v2_axis->get_state()  == Tango::ALARM)
	{
		new_state = Tango::ALARM;
	}

	if(h1_axis->get_state() == Tango::MOVING ||
		v1_axis->get_state() == Tango::MOVING ||
		h2_axis->get_state() == Tango::MOVING ||
		v2_axis->get_state()  == Tango::MOVING)
	{
		new_state = Tango::MOVING;
	}

	if(h1_axis->get_state() == Tango::FAULT ||
		v1_axis->get_state() == Tango::FAULT ||
		h2_axis->get_state() == Tango::FAULT ||
		v2_axis->get_state()  == Tango::FAULT)
	{
		new_state = Tango::FAULT;
	}

#else
	if(h1_axis->get_state() == Tango::FAULT ||
		v1_axis->get_state() == Tango::FAULT ||
		h2_axis->get_state() == Tango::FAULT ||
		v2_axis->get_state()  == Tango::FAULT)
	{
		set_state(Tango::FAULT);
		return;
	}
	if(h1_axis->get_state() == Tango::ALARM ||
		v1_axis->get_state() == Tango::ALARM ||
		h2_axis->get_state() == Tango::ALARM ||
		v2_axis->get_state()  == Tango::ALARM)
	{
		set_state(Tango::ALARM);
		return;
	}
	if(h1_axis->get_state() == Tango::MOVING ||
		v1_axis->get_state() == Tango::MOVING ||
		h2_axis->get_state() == Tango::MOVING ||
		v2_axis->get_state()  == Tango::MOVING)
	{
		set_state(Tango::MOVING);
		return;
	}
	if(h1_axis->get_state() == Tango::STANDBY &&
		v1_axis->get_state() == Tango::STANDBY &&
		h2_axis->get_state() == Tango::STANDBY &&
		v2_axis->get_state()  == Tango::STANDBY)
	{
		set_state(Tango::STANDBY);
		return;
	}
	if(h1_axis->get_state() == Tango::OFF &&
		v1_axis->get_state() == Tango::OFF &&
		h2_axis->get_state() == Tango::OFF &&
		v2_axis->get_state()  == Tango::OFF)
	{
		set_state(Tango::OFF);
		return;
	}
#endif

	set_state(new_state);
}

//--------------------------------------------------------
/**
 *	Execute the update_status command:
 */
//--------------------------------------------------------
void DsAxis::update_status()
{
	stringstream status_ss;

	status_ss << "H1: " << h1_axis->get_status() << endl;
	status_ss << "H2: " << h2_axis->get_status() << endl;

	status_ss << "V1: " << v1_axis->get_status() << endl;
	status_ss << "V2: " << v2_axis->get_status() << endl;

	set_status(status_ss.str());

}

//--------------------------------------------------------
/**
 *	Execute the exception_handler command:
 */
//--------------------------------------------------------
void DsAxis::exception_handler(Tango::DevFailed &ex, Tango::DevState new_state, const string base_msg)
{
	time(&last_exception_t);

	std::stringstream error_msg;

	error_msg << base_msg << " : " << endl;
    for (unsigned int i=0; i<ex.errors.length(); i++)
    {
            error_msg << ex.errors[i].reason.in() << endl;
            error_msg << ex.errors[i].desc.in() << endl;
            error_msg << ex.errors[i].origin.in() << endl;
    }
    error_msg << ends;

    ERROR_STREAM << error_msg.str() << endl;

    if(error_msg.str().find("API_WAttrOutsideLimit") != string::npos)
    	throw ex;

    set_state(new_state);
    set_status(error_msg.str());
}

	/*----- PROTECTED REGION END -----*/	//	DsAxis::namespace_ending
} //	namespace
