/*
 * DsAxisHandler.cpp
 *
 *  Created on: Mar 23, 2012
 *      Author: mdm
 */

#include "DsAxisHandler.h"
#include "DsAxis.h"
#include "ThreadGuard.h"

namespace DsAxis_ns
{

DsAxisHandler::DsAxisHandler(std::string axis_name, Tango::DevULong timeout, std::string positionAttr, std::string motionAccuracyAttr) throw(Tango::DevFailed)
{
	axis_proxy = new Tango::DeviceProxy(axis_name);
	axis_proxy->set_timeout_millis(timeout);

	axis_state = Tango::INIT;
	axis_status = "Initializing device";
	axis_initialised = 0;

	positionAttribute = positionAttr;
	motionAccuracyAttribute = motionAccuracyAttr;
}

DsAxisHandler::~DsAxisHandler()
{
	if(axis_proxy)
		delete axis_proxy;
}

void DsAxisHandler::update_attributes() throw(Tango::DevFailed)
{
	ThreadGuard lock(mutex);

#ifdef _MDM
	axis_state = axis_proxy->state();

	axis_status = axis_proxy->status();

	Tango::DeviceAttribute dev_attr = axis_proxy->read_attribute("IsInitialised");
	dev_attr >> axis_initialised;
#else
	Tango::DeviceAttribute dev_attr;
	dev_attr = axis_proxy->read_attribute("State"); dev_attr >> axis_state;
	dev_attr = axis_proxy->read_attribute("Status"); dev_attr >> axis_status;
	dev_attr = axis_proxy->read_attribute("IsInitialised");  dev_attr >> axis_initialised;
#endif
}

Tango::DevState DsAxisHandler::get_state()
{
	ThreadGuard lock(mutex);

	return axis_state;
}

std::string DsAxisHandler::get_status()
{
	ThreadGuard lock(mutex);

	return axis_status;
}

Tango::DevShort DsAxisHandler::is_initialised()
{
	ThreadGuard lock(mutex);

	return axis_initialised;
}

void DsAxisHandler::axis_init() throw(Tango::DevFailed)
{
	axis_proxy->command_inout("AxisInit");
}

void DsAxisHandler::axis_stop() throw(Tango::DevFailed)
{
	axis_proxy->command_inout("Stop");
}

void DsAxisHandler::motor_on() throw(Tango::DevFailed)
{
	axis_proxy->command_inout("MotorON");
}

void DsAxisHandler::motor_off() throw(Tango::DevFailed)
{
	axis_proxy->command_inout("MotorOFF");
}

Tango::DevDouble DsAxisHandler::read_position() throw(Tango::DevFailed)
{
	Tango::DevDouble position = 0;

	Tango::DeviceAttribute device_attr = axis_proxy->read_attribute(positionAttribute);
	device_attr >> position;

	return position;
}

void DsAxisHandler::write_position(Tango::DevDouble position) throw(Tango::DevFailed)
{
	Tango::DeviceAttribute device_attr(positionAttribute, position);
	axis_proxy->write_attribute(device_attr);
}

Tango::DevLong DsAxisHandler::get_motion_status() throw(Tango::DevFailed)
{
	Tango::DevLong motion_status = 0;

	Tango::DeviceData dev_data = axis_proxy->command_inout("AxisGetMotionStatus");
	dev_data >> motion_status;

	return motion_status;
}

Tango::DevDouble DsAxisHandler::get_motion_accuracy() throw(Tango::DevFailed)
{
	Tango::DevDouble motion_accuracy = 0;

  #if 0
	Tango::DbData dev_prop;
	dev_prop.push_back(Tango::DbDatum(motionAccuracyAttribute));

	axis_proxy->get_property(dev_prop);

	if (dev_prop[0].is_empty()==false)
		dev_prop[0]  >>  motion_accuracy;
  #else
	try {
		Tango::DeviceAttribute device_attr = axis_proxy->read_attribute(motionAccuracyAttribute);
		device_attr >> motion_accuracy;
	} catch (Tango::DevFailed e) {
		#ifdef _NO_EXCEPT_ON_ACCURACY
			motion_accuracy = 0.1;
		#else
			throw e;
		#endif
	} catch (...) {
		throw;
	}
  #endif

	return motion_accuracy;
}

}	//name space
